@@ -8,7 +8,7 @@
 #        .
 #          .
 #
-#	SCCS: @(#)Expression.pm	1.40 03/29/15 22:18:16
+#	SCCS: @(#)Expression.pm	1.44 03/30/15 18:29:00
 #
 # This module is free software; you can redistribute it and/or modify
 # it under the same terms as Perl itself. You must preserve this entire copyright
@@ -50,7 +50,7 @@ use vars qw/
 	$Version
 );
 
-our $VERSION = "1.40";
+our $VERSION = "1.44";
 
 # Fundamental to this is a tree of nodes.
 # Nodes are hashes with members:
@@ -148,6 +148,12 @@ my %InFunLV = map { $_ => 1} qw / defined pop push shift unshift /;
 
 # Default error output function
 sub PrintError {
+	my $self = shift;
+	if(defined $self->{PrintErrFunc} && $self->{PrintErrFunc}) {
+		$self->{PrintErrFunc}(@_);
+		return;
+	}
+
 	printf STDERR @_;
 	print STDERR "\n";
 }
@@ -163,19 +169,19 @@ sub VarSetFun {
 	my ($self, $name, @value) = @_;
 
 	unless(defined($name)) {
-		$self->{PrintErrFunc}("Undefined variable name '$name' - need () to force left to right assignment ?");
+		$self->PrintError("Undefined variable name '$name' - need () to force left to right assignment ?");
 	} else {
 		if($name =~ /^(.+)\[(\d+)\]$/) {
 			unless(defined($self->{VarHash}->{$1})) {
 				if($2 == 0) {
 					$self->{VarHash}->{$1} = $value[-1];
 				} else {
-					$self->{PrintErrFunc}("Can only create variable '%s' by setting element 0", $1);
+					$self->PrintError("Can only create variable '%s' by setting element 0", $1);
 				}
 			} elsif($2 > $self->{ArrayMaxIndex}) {
-				$self->{PrintErrFunc}("Array index %d is too large. Max is %d", $2, $self->{ArrayMaxIndex});
+				$self->PrintError("Array index %d is too large. Max is %d", $2, $self->{ArrayMaxIndex});
 			} elsif($2 > @{$self->{VarHash}->{$1}}) {
-				$self->{PrintErrFunc}("Extending array too much, '%s' has %d elements, trying to set element %d", $1, scalar @{$self->{VarHash}->{$1}}, $2);
+				$self->PrintError("Extending array too much, '%s' has %d elements, trying to set element %d", $1, scalar @{$self->{VarHash}->{$1}}, $2);
 		
 			} else {
 				$self->{VarHash}->{$1}[$2] = $value[-1];
@@ -264,11 +270,13 @@ sub ParseString {
 		elsif($expr =~ s/^(((\d+(\.\d*)?)|(\.\d+))([ed][-+]?\d+)?)//i) {
 			$newt->{oper} = 'const';
 			$newt->{val} = $1;
+			$newt->{type} = 'num';	# Used in debug/tree-print
 			$operlast = 0;
 		} # Match string bounded by ' or "
 		elsif($expr =~ /^(['"])/ and $expr =~ s/^$1([^$1]*)$1//) {
 			$newt->{oper} = 'const';
 			$newt->{val} = $1;
+			$newt->{type} = 'str';
 			$operlast = 0;
 		} elsif($expr =~ s/^}//) {
 			# Always need a ';' after this - magic one up to be sure
@@ -302,7 +310,7 @@ sub ParseString {
 		} # Match 'function(', leave '(' in input:
 		elsif($expr =~ s/^([_a-zA-Z][\w]*)\(/(/) {
 			unless($InFuns{$1}) {
-				$self->{PrintErrFunc}("Unknown function '%s'", $1);
+				$self->PrintError("Unknown function '%s'", $1);
 				return;
 			}
 			$newt->{oper} = 'func';
@@ -314,7 +322,7 @@ sub ParseString {
 			$newt->{name} = defined($1) ? $1 : defined($2) ? $2 : $3;
 			$operlast = 0;
 		} else {
-			$self->{PrintErrFunc}("Unrecognised input in expression at '%s'", $expr);
+			$self->PrintError("Unrecognised input in expression at '%s'", $expr);
 			return;
 		}
 
@@ -332,7 +340,7 @@ sub ParseString {
 			# End of input ?
 			if($EndInput and @operators == 0) {
 				if(@tree != 1) {	# There should be one node left - the root
-					$self->{PrintErrFunc}("Expression error - %s",
+					$self->PrintError("Expression error - %s",
 						$#tree == -1 ? "it's incomplete" : "missing operator");
 					return;
 				}
@@ -350,7 +358,7 @@ sub ParseString {
 			# Eliminate () - where current node is a close bracket
 			if($newt and @operators and $operators[-1]->{oper} eq '(' and $newt->{oper} eq ')') {
 				if($EndInput and $#operators != 0) {
-					$self->{PrintErrFunc}("Unexpected end of expression with unmatched '$operators[-1]->{oper}'");
+					$self->PrintError("Unexpected end of expression with unmatched '$operators[-1]->{oper}'");
 					return;
 				}
 
@@ -361,7 +369,7 @@ sub ParseString {
 			# Should have a new node to play with - unless end of string
 			if(!$newt && !$EndInput) {
 				if($loopb++ > 40) {
-					$self->{PrintErrFunc}("Internal error, infinite loop at: $expr");
+					$self->PrintError("Internal error, infinite loop at: $expr");
 					return;
 				}
 				next;
@@ -420,7 +428,7 @@ sub ParseString {
 					# ';' are special, don't need operands, also can lose empty ';' nodes
 					next if($op->{oper} eq ';' or $op->{oper} eq 'EOF');
 
-					$self->{PrintErrFunc}("Missing operand to operator '%s' at %s", $op->{oper},
+					$self->PrintError("Missing operand to operator '%s' at %s", $op->{oper},
 						($expr ne '' ? "'$expr'" : 'end'));
 
 					return;
@@ -464,28 +472,28 @@ sub CheckTreeInt {
 	my $ok = 1;
 
 	if(defined($MatchOp{$tree->{oper}}) or defined($MatchOpClose{$tree->{oper}})) {
-		$self->{PrintErrFunc}("Unmatched bracket '%s'", $tree->{oper});
+		$self->PrintError("Unmatched bracket '%s'", $tree->{oper});
 		$ok = 0;
 	}
 
 	if(defined($MonVarOp{$tree->{oper}}) and (!defined($tree->{right}) or ($tree->{right}{oper} ne '[' and $tree->{right}{oper} ne 'var'))) {
-		$self->{PrintErrFunc}("Operand to '%s' must be a variable or indexed array element", $tree->{oper});
+		$self->PrintError("Operand to '%s' must be a variable or indexed array element", $tree->{oper});
 		$ok = 0;
 	}
 
 	if($tree->{oper} eq '?' and $tree->{right}{oper} ne ':') {
-		$self->{PrintErrFunc}("Missing ':' operator after '?' operator") unless($ChkErrs);
+		$self->PrintError("Missing ':' operator after '?' operator") unless($ChkErrs);
 		$ok = 0;
 	}
 
 	if($tree->{oper} ne 'func') {
 		unless((!defined($tree->{left}) and defined($tree->{monop})) or $self->CheckTree($tree->{left})) {
-			$self->{PrintErrFunc}("Missing LH expression to '%s'", defined($tree->{monop}) ? $tree->{monop} : $tree->{oper}) unless($ChkErrs);
+			$self->PrintError("Missing LH expression to '%s'", defined($tree->{monop}) ? $tree->{monop} : $tree->{oper}) unless($ChkErrs);
 			$ok = 0;
 		}
 	}
 	unless(&CheckTree($self, $tree->{right})) {
-		$self->{PrintErrFunc}("Missing RH expression to '%s'", defined($tree->{monop}) ? $tree->{monop} : $tree->{oper}) unless($ChkErrs);
+		$self->PrintError("Missing RH expression to '%s'", defined($tree->{monop}) ? $tree->{monop} : $tree->{oper}) unless($ChkErrs);
 		$ok = 0;
 	}
 
@@ -493,7 +501,7 @@ sub CheckTreeInt {
 		my $fname = $tree->{fname};
 		if(($fname eq 'pop' or $fname eq 'shift' or $fname eq 'push' or $fname eq 'unshift') and
 		   (!defined($tree->{right}->{oper}) or (($tree->{right}->{oper} ne 'var' and $tree->{right}->{oper} ne ',') and (!defined($tree->{right}->{left}->{oper}) or $tree->{right}->{left}->{oper} ne 'var')))) {
-			$self->{PrintErrFunc}("First argument to $fname must be a variable");
+			$self->PrintError("First argument to $fname must be a variable");
 			$ok = 0;
 		}
 	}
@@ -617,7 +625,7 @@ sub EvalTree {
 			$index = $right[-1];
 
 			unless($index =~ /^-?\d+$/) {
-				$self->{PrintErrFunc}("Array '%s' index is not integer '%s'", $name, $index);
+				$self->PrintError("Array '%s' index is not integer '%s'", $name, $index);
 				return undef;
 			}
 
@@ -649,14 +657,14 @@ sub EvalTree {
 		my $right = $right[$#right];
 		unless(defined($right)) {
 			unless($self->{AutoInit}) {
-				$self->{PrintErrFunc}("Operand to mondaic operator '%s' is not defined", $oper);
+				$self->PrintError("Operand to mondaic operator '%s' is not defined", $oper);
 				return;
 			}
 			$right = 0;	# Monadics are all numeric
 		}
 
 		unless($right =~ /^([-+]?)0*([\d.]+)([ef][-+]?\d*|)$/i) {
-			$self->{PrintErrFunc}("Operand to monadic '%s' is not numeric '%s'", $oper, $right);
+			$self->PrintError("Operand to monadic '%s' is not numeric '%s'", $oper, $right);
 			return;
 		}
 		$right = "$1$2$3";
@@ -666,7 +674,7 @@ sub EvalTree {
 		return !$right if($oper eq '!');
 		return ~$right if($oper eq '~');
 
-		$self->{PrintErrFunc}("Unknown monadic operator when evaluating: '%s'", $oper);
+		$self->PrintError("Unknown monadic operator when evaluating: '%s'", $oper);
 		return;
 	}
 
@@ -705,12 +713,12 @@ sub EvalTree {
 		if($tree->{flow} eq 'while') {
 			my $ret = 0; # Return val, until get something better
 			if( !$self->{PermitLoops}) {
-				$self->{PrintErrFunc}("Loops not enabled, set property PermitLoops to do so");
+				$self->PrintError("Loops not enabled, set property PermitLoops to do so");
 				return;
 			}
 			while(1) {
 				if($self->{MaxLoopCount} && ++$self->{LoopCount} > $self->{MaxLoopCount}) {
-					$self->{PrintErrFunc}("Loop exceeded maximum iterations: MaxLoopCount = $self->{MaxLoopCount}");
+					$self->PrintError("Loop exceeded maximum iterations: MaxLoopCount = $self->{MaxLoopCount}");
 					return;
 				}
 				# left is loop condition, right is body:
@@ -726,9 +734,9 @@ sub EvalTree {
 	# Take care to avoid evaluating a tree twice, not just inefficient but nasty side effects with ++ & -- operators
 	my @left = $self->EvalTree($tree->{left}, $wantlv);
 	my $left = $left[$#left];
-	if(!defined($left) and $oper ne ',' and $oper ne '.') {
+	if(!defined($left) and $oper ne ',' and $oper ne '.' and $oper ne ';') {
 		unless($self->{AutoInit}) {
-			$self->{PrintErrFunc}("Left value to operator '%s' is not defined", $oper);
+			$self->PrintError("Left value to operator '%s' is not defined", $oper);
 			return;
 		}
 		$left = '';	# Set to the empty string
@@ -758,11 +766,14 @@ sub EvalTree {
 	return (@left, @right)	if($oper eq ',');
 	return @right		if($oper eq ';');
 
-	# Array index
+	# Array index. Beware: works differently depending on $wantlv.
+	# Because when $wantlv it is the array name, not its contents
 	if($oper eq '[') {
 		return undef	# Check if the array member could exist; ie have index
 			if($right !~ /^-?\d+$/);
 
+		@left = $self->{VarGetFun}($self, $left[0]) if($wantlv);
+
 		my $index = $right[-1];
 		$index += @left if($index < 0);	# Convert -ve index to a +ve one
 
@@ -784,7 +795,7 @@ sub EvalTree {
 		$left = ""  unless(defined($left));
 		$right = "" unless(defined($right));
 		if(length($left) + length($right) > $self->{StringMaxLength}) {
-			$self->{PrintErrFunc}("Joined string would exceed maximum allowed %d", $self->{StringMaxLength});
+			$self->PrintError("Joined string would exceed maximum allowed %d", $self->{StringMaxLength});
 			return "";
 		}
 		return $left . $right;
@@ -792,7 +803,7 @@ sub EvalTree {
 
 	unless(defined($right)) {
 		unless($self->{AutoInit}) {
-			$self->{PrintErrFunc}("Right value to operator '%s' is not defined", $oper);
+			$self->PrintError("Right value to operator '%s' is not defined", $oper);
 			return;
 		}
 		$right = '';
@@ -813,7 +824,7 @@ sub EvalTree {
 	# Perl would treat 012 as an octal number, that would confuse most people, convert to a decimal interpretation.
 	unless($left =~ /^([-+]?)0*([\d.]+)([ef][-+]?\d*|)/i) {
 		unless($self->{AutoInit} and $left eq '') {
-			$self->{PrintErrFunc}("Left hand operator to '%s' is not numeric '%s'", $oper, $left);
+			$self->PrintError("Left hand operator to '%s' is not numeric '%s'", $oper, $left);
 			return;
 		}
 		$left = 0;
@@ -823,7 +834,7 @@ sub EvalTree {
 
 	unless($right =~ /^([-+]?)0*([\d.]+)([ef][-+]?\d*|)/i) {
 		unless($self->{AutoInit} and $right eq '') {
-			$self->{PrintErrFunc}("Right hand operator to '%s' is not numeric '%s'", $oper, $right);
+			$self->PrintError("Right hand operator to '%s' is not numeric '%s'", $oper, $right);
 			return;
 		}
 		$right = 0;
@@ -847,7 +858,7 @@ sub EvalTree {
 	return $left != $right ? 1 : 0 if($oper eq '!=');
 	return $left != $right ? 1 : 0 if($oper eq '<>');
 
-	$self->{PrintErrFunc}("Unknown operator when evaluating: '%s'", $oper);
+	$self->PrintError("Unknown operator when evaluating: '%s'", $oper);
 	return;
 }
 
@@ -873,12 +884,12 @@ sub FuncValue {
 	# Beware: could exceed max length with: printf("%2000s", "foo");
 	if($fname eq 'printf') {
 		unless($self->{EnablePrintf}) {
-			$self->{PrintErrFunc}("Function 'printf' not enabled");
+			$self->PrintError("Function 'printf' not enabled");
 			return "";
 		}
 		my $s = sprintf $arglist[0], @arglist[1 ... $#arglist];
 		return $s if(length($s) <= $self->{StringMaxLength});
-		$self->{PrintErrFunc}("String would exceed maximum allowed %d", $self->{StringMaxLength});
+		$self->PrintError("String would exceed maximum allowed %d", $self->{StringMaxLength});
 		return "";
 	}
 
@@ -921,7 +932,7 @@ sub FuncValue {
 		return -1;
 	}
 
-	$self->{PrintErrFunc}("Unknown function '$fname'");
+	$self->PrintError("Unknown function '$fname'");
 
 	return '';
 }
@@ -933,7 +944,7 @@ sub new {
 
 	# What we store about this evaluation environment, default values:
 	my %ExprVars = (
-		PrintErrFunc	=>	\&PrintError,		# Printf errors
+		PrintErrFunc	=>	'',			# Printf errors
 		VarHash		=>	{(			# Variable hash
 					EmptyArray	=>	[()],
 					EmptyList	=>	[()],
@@ -965,8 +976,8 @@ sub SetOpt {
 	my $self = shift @_;
 
 	while($#_ > 0) {
-		$self->{PrintErrFunc}("Unknown option '$_[0]'") unless(exists($self->{$_[0]}));
-		$self->{PrintErrFunc}("No value to option '$_[0]'") unless(defined($_[1]));
+		$self->PrintError("Unknown option '$_[0]'") unless(exists($self->{$_[0]}));
+		$self->PrintError("No value to option '$_[0]'") unless(defined($_[1]));
 		$self->{$_[0]} = $_[1];
 		shift;shift;
 	}
@@ -1107,7 +1118,7 @@ Default: 1000.
 
 This is a printf style function that will be called in the event of an error,
 the error text will not have a trailing newline.
-The default is C<printf STDERR>.
+If this is not set the default is to C<printf STDERR>.
 
 =item VarHash
 
@@ -1510,7 +1521,7 @@ Alain D D Williams <addw@phcomp.co.uk>
 
 =head2 Copyright and Version
 
-Version "1.40", this is available as: $Math::Expression::Version.
+Version "1.44", this is available as: $Math::Expression::Version.
 
 Copyright (c) 2003, 2015 Parliament Hill Computers Ltd/Alain D D Williams. All rights reserved.
 This program is free software; you can redistribute it and/or modify it
